#include <fcntl.h>
#include <sys/select.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

#include "pair.h"

int active(struct in_addr *hostlist, int numhosts, int port, int timeout, struct in_addr *peer) {
	int sockd;
	struct sockaddr_in serv_name;
	int status;
	int i=0;
	time_t now;

	struct timeval confirm_timeout; /* timeout to confirm a connection in seconds */
	socklen_t addrlen;
	fd_set rset;
	char proto[5];

	if (numhosts<1) {
		printf("active: No peers found!\n");
		return(-1);
	}

	now=time(NULL);
	/* some low level of randomness to start with */
	i=now % numhosts;
	while (now+timeout>time(NULL)) {
		/* create a socket */
		sockd = socket(AF_INET, SOCK_STREAM, 0);
		if (sockd == -1)
		{
			perror("active: tcp socket creation error");
			return(-1);
		}

		/* server address */ 
		serv_name.sin_family = AF_INET;
		serv_name.sin_addr = hostlist[i];
		serv_name.sin_port = htons(port);

		/* connect to the server */
		printf("Trying to pair with %s...\n",inet_ntoa(hostlist[i]));
		status = connect(sockd, (struct sockaddr*)&serv_name, sizeof(struct sockaddr_in));
		if (status == -1) {
			/* connection failed, try the next guy on the list */
			sleep(2);
			i++;
			i%=numhosts;
		} else {
			/* test if we're the Chosen One on the other side */
			peer->s_addr=serv_name.sin_addr.s_addr;
			printf("Connected to %s, waiting for callback\n",inet_ntoa(*peer));
			addrlen = sizeof(struct sockaddr_in);
			FD_ZERO(&rset);
			FD_SET(sockd,&rset);
			confirm_timeout.tv_sec=1;
			confirm_timeout.tv_usec=0;
			status=select(sockd+1,&rset,NULL,NULL,&confirm_timeout);
			if(status == -1) {
				/* FAIL */
				perror("callback failed: select()");
			} else {
				printf("Connected... ");
				bzero(&proto,5);
				status=fcntl(sockd,F_SETFL,O_NONBLOCK);
				if (status == -1)
				{
					perror("active: fcntl()");
					exit(1);
				}
				if(0<read(sockd,&proto,5)) {
					printf("callback received: %s\n",proto);
					/* WIN */
					return(1);
				} else {
					printf("callback not received, trying another host\n");
					close(sockd);
					sleep(2);
					i++;
					i%=numhosts;
				}
			}
		}
	}

	close(sockd);
	return(-1);
}


