#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/ip_icmp.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "pair.h"

#define SEND_SIZE 64
#define RECV_SIZE 84

int build_host_list(struct in_addr* hostlist, int *numhosts, const char *maddr, int timeout) {
	int raw_socket;
	struct sockaddr_in dst;
	struct icmp *icmp;
	void *recvbuf;
	struct ip *recvaddr;
	int status;
	struct timeval listen_timeout;
	fd_set rset;
	time_t now;

	*numhosts=0;

	printf("building host list... timeout %d\n",timeout);
	
	raw_socket = socket(PF_INET, SOCK_RAW, IPPROTO_ICMP);
	if (raw_socket == -1) {
			perror("raw socket() (are you root?)");
			exit(1);
	}

	icmp = (struct icmp *)malloc((size_t)SEND_SIZE);

	memset(icmp,0,SEND_SIZE);

	icmp->icmp_type = ICMP_ECHO;
	icmp->icmp_code = 0;
	icmp->icmp_seq=htons(1);
	icmp->icmp_cksum=0xfef7; /* :-) network byte order too */

	dst.sin_family=AF_INET;
	dst.sin_addr.s_addr=inet_addr(maddr);
	dst.sin_port=0;
	status=sendto(raw_socket,icmp,SEND_SIZE,0,(struct sockaddr *)&dst,sizeof(struct sockaddr_in));
	free(icmp);
	if(0>status) { perror("send"); exit(1);}

	/* wait for replies */
	FD_ZERO(&rset);
	FD_SET(raw_socket,&rset);
	listen_timeout.tv_sec=1;
	listen_timeout.tv_usec=0;
	
	recvbuf=malloc((size_t)RECV_SIZE);
	if(NULL==recvbuf) { perror("malloc recvbuf"); exit(1); }
	memset(recvbuf,0,RECV_SIZE);

	/* spend WAIT_TIME seconds looking for reply packets */
	/* and do not block if we don't receive anything */
	now=time(NULL);
	while(((*numhosts)<MAX_HOSTS) && (now+timeout > time(NULL)) ) {
		status=select(raw_socket+1,&rset,NULL,NULL,&listen_timeout);
		if(status==-1) {
		   perror("select"); exit(1); 
		} else {
			if (FD_ISSET(raw_socket, &rset)) {
				status=recv(raw_socket,recvbuf,RECV_SIZE,0);
				if(0>status) { perror("recv"); exit(1); }

				/* ip header parsing :-P */
				recvaddr=recvbuf;
				/* sort out ICMP only */
				if (recvaddr->ip_p == 1) {
					printf("ICMP reply from: %s\n",inet_ntoa(recvaddr->ip_src));
					hostlist[(*numhosts)++]=recvaddr->ip_src;
				}
			}
		}
	}

	free(recvbuf);
	close(raw_socket);
	return 0;
}
